%FreakLab - FreakLab.m
%
%Erstellt von Matthias Hasewend im Rahmen der bung 'Applikationssoftware
%fr Fortgeschrittene' bei Ass. Prof. Winfried Kernbichler, Institut fr
%Theoretische Physik / Computational Physics an der TU Graz im WS 2010/11.
%
%Beschreibung:
%
%Dieses Matlab-GUI dient zur Bearbeitung von Bildern. Es knnen damit
%nderungen an der Bildgeometrie und dem rgb-Farbwert an jedem Pixel 
%vorgenommen werden. Dazu stehen vier verschiedene Bedienfelder zur
%Verfgung:
%
%1) Lineare Abbildung
%2) Holomorphe Abbildung
%3) Allgemeine komplexe Abbildung
%4) Farbabbildung
%
%Die Callback-Funktionen der Bedienelemente dieser Felder finden sich in
%den Skript-Files FreakLab_opt1 bis FreakLab_opt4. Darin sind auch
%Erklrungen der Funktionsweise enthalten. Neben diesen 4 Files gibt es
%noch 22 weitere .m-Dateien mit Hilfsfunktionen, die in den Callbacks
%bentigt werden.
%
%Dieses Hauptfile stellt die zentrale Verwaltungseinheit des Programms dar.
%Es ermglicht das ffnen und Speichern einer Datei sowie die Auswahl und 
%Anzeige eines Bedienfeldes. Fr die im Bedienfeld festgelegten
%Transformationen des Bildes erscheint ein 30 x 30 Pixel Preview-Feld.
%Diese Transformationen knnen auf die Bilddatei, welche im groen 
%Anzeigefeld dargestellt ist, angewandt werden; die Anwendung lsst sich
%rckgngig machen oder kann bernommen werden. Mit Reset wird das gesamte
%Programm auf den Ausgangszustand zurckgesetzt.
%
%Das Herzstck des gesamten Programmes ist die Matlab-Routine imtransform
%fr die geometrischen Transformationen. Fr die Farbabbildung wurde ein
%imtransform_eigen entwickelt (siehe imtransform_eigen.m).

%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

%Vom Matlab-Guide generierter Initialisationscode
function varargout = FreakLab(varargin)

    gui_Singleton = 1;
    
    gui_State = struct('gui_Name',       mfilename, ...
                       'gui_Singleton',  gui_Singleton, ...
                       'gui_OpeningFcn', @FreakLab_OpeningFcn, ...
                       'gui_OutputFcn',  @FreakLab_OutputFcn, ...
                       'gui_LayoutFcn',  [] , ...
                       'gui_Callback',   []);
    if(nargin && ischar(varargin{1}))
        
        gui_State.gui_Callback = str2func(varargin{1});
        
    end

    if(nargout)
        
        [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
        
    else
        
        gui_mainfcn(gui_State, varargin{:});
        
    end
    
end

%Startup-Funktion des Programms
function FreakLab_OpeningFcn(hObject,eventdata,handles,varargin)

    %Maximale ffungsgre eines Bildes
    handles.limsize = 2000*1000;
    %Abspeicherqualitt: 100 ~> 100% ~> maximale Qualitt
    handles.qual = 100;
    
    %Initialisierungsfunktion: Weist den Bedienelementen ihre Callback- und
    %KeyPress-Funktionen zu
    handles = init(hObject,handles);

    %Nummer des gerade angezeigten Bedienfeldes
    handles.toggle = 1;

    %Erzeugt die globalen Variablen, die allesamt in der handles enthalten
    %sind, fr das Hauptprogramm ...
    handles = handleset(hObject,handles,0);
    %... und fr das Bedienfeld 1, das beim Start defaultmig erscheint
    handles = handleset(hObject,handles,1);

    %Deaktiviert das Bedienfeld zu Beginn, da keine Datei geffnet
    disenable(hObject,handles,'off',1);

    %bernehmen der in dieser Unterfunktion gettigten nderungen der
    %Struktur handles
    guidata(hObject,handles);

end


%Endup-Funktion des Programms
function varargout = FreakLab_OutputFcn(hObject,eventdata,handles) 

    varargout{1} = handles.output;
    
end

%Aktion beim Drcken von 'ffnen'
function pushbutton6_Callback(hObject, eventdata, handles)

    %JPG Datei aus dem Default-Ordner auswhlen
    [name,source] = uigetfile({'*.jpg','Bilder (*.jpg)'},'Bild auswhlen');

    %Dateiname mit Pfad abspeichern
    handles.imname = [source,name];
    
    %Falls tatschlich eine Datei ausgewhlt wurde...
    if(name ~= 0)

        %Einlesen der Datei
        handles.image = imread(handles.imname);
        %Gre merken
        handles.imsize = size(handles.image);
        
        %Bildgre muss unter Grenzgre liegen
        if((prod(handles.imsize)/3) >= handles.limsize)
            
            beep;
            errordlg('Dieses Bild ist zu gro.','Problem');
            
        else
        
            %Erzeugen des Preview-Bildchens
            handles.imtiny = imresize(handles.image,[30,30]);

            %Anzeigen des Bildnamens
            set(handles.edit15,'String',name);

            %Bild anzeigen
            set(gcf,'CurrentAxes',handles.axes1);
            image(handles.image);
            axis off;

            %Preview-Bildchen anzeigen
            set(gcf,'CurrentAxes',handles.axes2);
            image(handles.imtiny);
            axis off;
            
            %Handes fr das aktuelle Bedienfeld neu erzeugen (auffrischen)
            handles = handleset(hObject,handles,handles.toggle);
            %Aktuelles Bedienfeld aktiviern
            disenable(hObject,handles,'on',handles.toggle);
            
            guidata(hObject,handles);
            
        end

    end

end

%Aktion beim Drcken von 'Speichern'
function pushbutton7_Callback(hObject,eventdata,handles)

    name = handles.imname;

    siz = 1:length(name);
    siz = siz(name == '.');
    siz = siz(end);
    
    %Name um '_mod_' fr 'modifiziert' erweitern
    name = [name(1:(siz-1)),'_mod',name(siz:end)];

    %Speicherort auswhlen
    [name,source] = uiputfile({'*.jpg','Bilder (*.jpg)'},'Ort auswhlen',name);

    %Falls tatschlich ein Ort ausgewhlt wurde...
    if(name ~= 0)
        
        %Schreiben der Datei an den gewnschten Ort mit der gewnschten
        %Qualittsstufe
        imwrite(handles.image,[source,name],'jpg','Quality',handles.qual);

    end

end

%Aktion beim Drcken von 'Reset'
function pushbutton8_Callback(hObject,eventdata,handles)

    %Beide Anzeigefelder werden geleert
    set(gcf,'CurrentAxes',handles.axes1);
    cla reset;
    axis off;
    
    set(gcf,'CurrentAxes',handles.axes2);
    cla reset;
    axis off;
    
    %Aktuelles Bedienfeld wird deaktiviert
    disenable(hObject,handles,'off',handles.toggle);

    %Name wird leer
    handles.imname = '';
    set(handles.edit15,'String','');
    
    %Handles fr aktuelles Bedienfeld werden aufgefrischt
    handles = handleset(hObject,handles,handles.toggle);
    
    guidata(hObject,handles);

end

%Aktion beim Auswahl des 1. Bedienfeldes
function togglebutton1_Callback(hObject,eventdata,handles)

    %Aufruf einer Hilfsfunktion
    handles = toggleoff(hObject,handles,1,handles.toggle);
    %Welches Bedienfeld ist ausgewhlt?
    handles.toggle = 1;
    %Zugehriger Togglebutton auf 'gedrckt' gesetzt
    set(hObject,'Value',1);
    %Bedienfeld wird sichtbar
    set(handles.uipanel3,'Visible','on');
    guidata(hObject,handles);

end

%Aktion beim Auswahl des 2. Bedienfeldes
function togglebutton2_Callback(hObject,eventdata,handles)

    handles = toggleoff(hObject,handles,2,handles.toggle);
    handles.toggle = 2;
    set(hObject,'Value',1);
    set(handles.uipanel5,'Visible','on');
    guidata(hObject,handles);

end

%Aktion beim Auswahl des 3. Bedienfeldes
function togglebutton3_Callback(hObject,eventdata,handles)

    handles = toggleoff(hObject,handles,3,handles.toggle);
    handles.toggle = 3;
    set(hObject,'Value',1);
    set(handles.uipanel6,'Visible','on');
    guidata(hObject,handles);
    
end

%Aktion beim Auswahl des 4. Bedienfeldes
function togglebutton4_Callback(hObject,eventdata,handles)

    handles = toggleoff(hObject,handles,4,handles.toggle);
    handles.toggle = 4;
    set(hObject,'Value',1);
    set(handles.uipanel7,'Visible','on');
    guidata(hObject,handles);
    
end

%Hilfsfunktion fr Bedienfeldauswahl
function handles = toggleoff(hObject,handles,new,old)

    %Altes Bedienfeld deaktivieren
    disenable(hObject,handles,'off',old);

    %Alle Buttons auf 'nicht gedrckt' gesetzt
    set(handles.togglebutton1,'Value',0);
    set(handles.togglebutton2,'Value',0);
    set(handles.togglebutton3,'Value',0);
    set(handles.togglebutton4,'Value',0);
    
    %Alle Bedienfelder verbergen
    set(handles.uipanel3,'Visible','off');
    set(handles.uipanel5,'Visible','off');
    set(handles.uipanel6,'Visible','off');
    set(handles.uipanel7,'Visible','off');
    
    %Handles fr das neue Bedienfeld auffrischen
    handles = handleset(hObject,handles,new);
    
    if(isempty(handles.imname))
        
        %Falls keine Datei geladen: Neues Feld erscheint deaktiviert
        disenable(hObject,handles,'off',new);
        
    else

        %Neues Feld erscheint aktiviert
        disenable(hObject,handles,'on',new);
        
    end
    
end

%Aktion beim Drcken von 'Anwenden'
function pushbutton1_Callback(hObject,eventdata,handles)

    apply(hObject,eventdata,handles,handles.toggle);

end

%Aktion beim Drcken von 'Rckgngig'
function pushbutton2_Callback(hObject,eventdata,handles)

    undo(hObject,eventdata,handles,handles.toggle);

end

%Aktion beim Drcken von 'Okay'
function pushbutton3_Callback(hObject,eventdata,handles)

    confirm(hObject,eventdata,handles,handles.toggle);

end

%Aktion beim Drcken von 'Info'
function pushbutton13_Callback(hObject,eventdata,handles)

    %Info-Text
    string = ['Dieses Programm wurde erstellt von Matthias Hasewend im Rahmen '];
    string = [string,'der bung ''Applikationssoftware fr Fortgeschrittene'' '];
    string = [string,'bei Ass. Prof. Winfried Kernbichler, Institut fr Theoretische '];
    string = [string,'Physik / Computational Physics, TU Graz, WS 2010/11.'];

    %Button deaktivieren, damit er nicht stndig gedrck werden kann und
    %sich daher immer mehr Info-Boxen ffen
    set(hObject,'Enable','Off');
    
    %Box ffnen
    boxhand = msgbox(string,'Info','help');
    
    %Solange Box offen ist, also  get etwas zurckliefert, soll...
    while(1)
        
        try
            
            boxget = get(boxhand);
            pause(0.1);
            
        catch
            
            break;
            
        end
    end
    
    %... der Button inaktiv bleiben
    set(hObject,'Enable','On');

end