%FreakLab - FreakLab_opt2.m
%
%Erstellt von Matthias Hasewend im Rahmen der bung 'Applikationssoftware
%fr Fortgeschrittene' bei Ass. Prof. Winfried Kernbichler, Institut fr
%Theoretische Physik / Computational Physics an der TU Graz im WS 2010/11.
%
%Beschreibung:
%
%Dieses Skript steuert das Bedienfeld 2 - Holomorphe Abbildung.
%
%Theorie:
%
%In FreakLab_opt1.m entspricht jedem Pixel eines Bildes ein Element eines 
%2D-Vektorraumes. Hier werden den Pixeln Elemente der komplexen Zahlenebene
%zugewiesen. Abbildungen zwischen komplexen Zahlenebenen sind komplexe
%Funktionen. Darunter zeichnen sich die differenzierbaren Funktionen
%besonders aus. Sie fhren zwar keine Linearen Abbildungen mehr aus, jedoch
%folgt aus der Stetigkeit des Urbildes die (fast-berall-) Stetigkeit des 
%Bildes. Man nennt solche Funktionen auch holomorph. Dieses Skript fhrt 
%also nunmehr nichtlineare aber weiterhin stetige Bildtransformationen mit 
%interessanten Resultaten aus.
%
%Eine holomorphe Funktion ist immer unendlich oft differenzierbar und lsst
%sich durch Potenzreihen darstellen (Laurent-Reihen), also als Summe von
%Polynomen mit unterschiedlichen Exponenten. Hier kann jede holomorphe
%Funktion in der Weise approximiert werden, dass bis zu 9 Summanden als
%Polynome mit unterschiedlichem Vorfaktor, Nullstelle und Exponent
%eingegeben werden knnen.
%
%Der Exponent darf darber hinaus sogar nichtganzzahlig sein. Auerdem
%knnen der Urbild- und der Bild-Zoomfaktor ber Slider und Textboxen
%eingegeben werden. Desweiteren kann auf ein Bild umgeschalten werden,
%das sich aus einem Urbild ergibt, das periodisch fortgesetzt wird
%(checkbox4)

%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

%Diese Funktion bewirkt, dass das alle anderen Funktionen in diesem Skript 
%von aussen aufgerufen werden knnen in der Form: FreakLab_opt2('Funkname')
function varargout = FreakLab_opt2(varargin)

    func = str2func(varargin{1});
    
    if(nargout)
        
        [varargout{1:nargout}] = func(varargin{2:end});
        
    else
        
        func(varargin{2:end});
        
    end
    
end

%Aktion beim Einstellen der 'Urbild-Zoomstrke' ber den Slider
function slider5_Callback(hObject, eventdata, handles)

    inp = get(hObject,'Value');
    
    handles.uv = inp*1.5 + 0.5;
    
    %Wert in der geeigneten Genauigkeit in das Textfeld setzen
    set(handles.edit13,'String',num2str(handles.uv,'%1.2f'));
    
    disp_2_1(hObject,handles);
    
end

%Aktion beim Einstellen der 'Bild-Zoomstrke' ber den Slider
function slider7_Callback(hObject, eventdata, handles)

    inp = get(hObject,'Value');
    
    handles.xy = inp*1.5 + 0.5;
    
    set(handles.edit14,'String',num2str(handles.xy,'%1.2f'));
    
    disp_2_1(hObject,handles);
    
end

%Man beachte: Alle Edit-Textfelder wurden unter init mit der
%KeyPressFcn editupdate versehen. Diese bewirkt ein Update des
%Previews schon whrend der Eingabe (siehe editupdate.m)

%Aktion bei Eingabe der 'Urbild-Zoomstrke' ber das Textfeld
function edit13_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(1,['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');

        inp = inpcheck(str2num(inp),1,0.5,2);
        
        handles.uv = inp;
        len = (len >= 3) + (len >= 4);
        
        %Slider angleichen
        set(handles.slider5,'Value',(inp - 0.5)/1.5);
        
        %Wert in der geeigneten Genauigkeit in das Textfeld setzen
        set(hObject,'String',num2str(inp,['%1.',num2str(len),'f']));
        
        disp_2_1(hObject,handles);
    
    end

end

%Aktion bei Eingabe der 'Bild-Zoomstrke' ber das Textfeld
function edit14_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(1,['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');

        inp = inpcheck(str2num(inp),1,0.5,2);
        
        handles.xy = inp;
        len = (len >= 3) + (len >= 4);
        
        set(handles.slider7,'Value',(inp - 0.5)/1.5);
        
        set(hObject,'String',num2str(inp,['%1.',num2str(len),'f']));
        
        disp_2_1(hObject,handles);
    
    end
    
end

%Aktion bei Eingabe des Realteils der Nullstelle des aktuellen Polynoms
%ber das Textfeld
function edit9_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(0,['%1.3f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');

        inp = inpcheck(str2num(inp),0,-1,1);
        
        handles.holtra(handles.holtrac,2) = inp;
        len = (len >= 3) + (len >= 4) + (len >= 5);
        
        set(hObject,'String',num2str(inp,['%1.',num2str(len),'f']));
        
        disp_2_1(hObject,handles);
    
    end

end

%Aktion bei Eingabe des Imaginrteils der Nullstelle des aktuellen Polynoms
%ber das Textfeld
function edit10_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(0,['%1.3f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');

        inp = inpcheck(str2num(inp),0,-1,1);
        
        handles.holtra(handles.holtrac,3) = inp;
        len = (len >= 3) + (len >= 4) + (len >= 5);
        
        set(hObject,'String',num2str(inp,['%1.',num2str(len),'f']));
        
        disp_2_1(hObject,handles);
    
    end

end
%Anmerkung: Real- und Imaginrteil knnen auch durch Klicken auf das groe
%Anzeigefeld eingegeben werden (siehe motion.m).

%Aktion bei Eingabe des Exponenten des aktuellen Polynoms ber das Textfeld
function edit11_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(1,['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');

        inp = inpcheck(str2num(inp),1,-10,10);
        
        handles.holtra(handles.holtrac,4) = inp;
        len = (len >= 3) + (len >= 4);
        
        set(hObject,'String',num2str(inp,['%1.',num2str(len),'f']));
        
        set(handles.slider6,'Value',(inp + 10)/20);
        
        disp_2_1(hObject,handles);
    
    end

end

%Aktion bei Eingabe des Vorfaktors des aktuellen Polynoms ber das Textfeld
function edit12_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
    set(hObject,'String',num2str((handles.holtrac == 1),['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');

        %Die sonderbare Logik-Konstruktion stellt sicher, dass der erste
        %Summand einen nichtverschwindenden Vorfaktor besitzt, alle anderen
        %Vorfaktoren aber durchaus 0 sein drfen
        inp = inpcheck(str2num(inp),1,0.5*(handles.holtrac == 1) - 3*(handles.holtrac ~= 1),3);
        
        handles.holtra(handles.holtrac,1) = inp;
        len = (len >= 3) + (len >= 4);
        
        set(hObject,'String',num2str(inp,['%1.',num2str(len),'f']));
        
        disp_2_1(hObject,handles);
    
    end

end

%Aktion beim Einstellen des Exponenten des aktuellen Polynoms ber den Slider
function slider6_Callback(hObject, eventdata, handles)

    inp = get(hObject,'Value');
    
    handles.holtra(handles.holtrac,4) = inp*20 - 10;
    
    set(handles.edit11,'String',num2str(handles.holtra(handles.holtrac,4),'%1.1f'));
    
    disp_2_1(hObject,handles);
    
end

%Aktion beim Setzen der Checkbox fr periodische Fortsetzung
function checkbox4_Callback(hObject, eventdata, handles)

    handles.per = get(hObject,'Value');
    
    disp_2_1(hObject,handles);
    
end

%Aktion bei Auswahl das aktuell angezeigten Summanden (Polynom)
function popupmenu1_Callback(hObject, eventdata, handles)
    
    %Nummer des Summanden, der angezeigt werden soll
    handles.holtrac = get(hObject,'Value');
    
    %Darstellung von Vorfaktor, Exponenten und Nullstelle des gewhlten
    %Summanden
    set(handles.edit12,'String',num2str(handles.holtra(handles.holtrac,1),'%1.2f'));
    set(handles.edit11,'String',num2str(handles.holtra(handles.holtrac,4),'%1.1f'));
    set(handles.edit9,'String',num2str(handles.holtra(handles.holtrac,2),'%1.3f'));
    set(handles.edit10,'String',num2str(handles.holtra(handles.holtrac,3),'%1.3f'));

    %Exponentenslider auf gewhlten Wert gestellt
    set(handles.slider6,'Value',(handles.holtra(handles.holtrac,4) + 10)/20);
    
    guidata(hObject,handles);

end

%Aktion bei Hinzufgen eines weiteren Summanden (Polynoms)
function pushbutton4_Callback(hObject, eventdata, handles)

    %Es muss mindestens einen Summanden geben, sobald nur ein weiterer
    %erstellt wird, kann aber der Button zum entfernen von Summanden
    %aktiviert werden
    set(handles.pushbutton5,'Enable','on');

    %Summandenzhler erhhen
    handles.trafos = handles.trafos + 1;

    %Im Summandenauswahl-Popupmenu wird ein weiterer Eintrag erstellt
    stri = get(handles.popupmenu1,'String');
    stri = [stri;['Summand ',num2str(handles.trafos)]];
    set(handles.popupmenu1,'String',stri);

    %Wenn die Maximalanzahl von 9 Summanden erreicht ist, wird der Button
    %zum Erstellen weiterer Summanden deaktiviert
    if(handles.trafos == 9)
        
        set(hObject,'Enable','off');
        
    end
    
    guidata(hObject,handles);

end

%Aktion bei Verringerung der Summe um einen Summanden (Polynom)
function pushbutton5_Callback(hObject, eventdata, handles)

    %Es darf maximal 9 Summanden geben, sobald nur einer davon gelscht
    %wird, kann aber der Button zum Hinzufgen von Summanden aktiviert 
    %werden
    set(handles.pushbutton4,'Enable','on');

    
    %Nach einem Lschvorgang wird der allererste Summand angezeigt
    handles.holtrac = 1;
    set(handles.popupmenu1,'Value',1);
    set(handles.edit11,'String',num2str(handles.holtra(1,4),'%1.1f'));
    set(handles.edit9,'String',num2str(handles.holtra(1,2),'%1.3f'));
    set(handles.edit10,'String',num2str(handles.holtra(1,3),'%1.3f'));
    set(handles.edit12,'String',num2str(handles.holtra(handles.holtrac,1),'%1.2f'));
    set(handles.slider6,'Value',(handles.holtra(1,4) + 10)/20);
    
    
    %Sofern es mehr als einen Summanden gibt, wird...
    if(handles.trafos >= 2)
        
        %...der Popupmenu-Auswahllistenstring geladen...
        stri = get(handles.popupmenu1,'String');
        
        %...,um 1 verkrzt...
        stri = stri(1:(end-1),:);
        
        handles.holtra(handles.trafos,:) = [0,0,0,1];
        
        handles.trafos = handles.trafos - 1;
        
        %....und wieder ins Men eingefgt.
    
        set(handles.popupmenu1,'String',stri);
        
    end

    %Wenn die Minimalanzahl von 1 Summand erreicht ist, wird der Button
    %zum Lschen von Summanden deaktiviert
    if(handles.trafos == 1)
        
        set(hObject,'Enable','off');
        
    end
    
    %Anzeige des Previews mit der verkrzten Summe
    disp_2_1(hObject,handles);
    
    guidata(hObject,handles);

end